package com.ruoyi.ems.mapper;

import java.util.List;

import com.ruoyi.ems.domain.EmsDevice;
import com.ruoyi.ems.domain.vo.QueryDevice;

/**
 * 能源设备Mapper接口
 *
 * @author ruoyi
 * @date 2024-07-10
 */
public interface EmsDeviceMapper {
    /**
     * 查询能源设备
     *
     * @param id 能源设备主键
     * @return 能源设备
     */
    EmsDevice selectEmsDeviceById(Long id);

    /**
     * 查询能源设备
     * @param code 能源设备编码
     * @return 能源设备
     */
    EmsDevice selectEmsDeviceByCode(String code);

    /**
     * 查询能源设备列表
     *
     * @param emsDevice 能源设备
     * @return 能源设备集合
     */
    List<EmsDevice> selectEmsDeviceList(EmsDevice emsDevice);

    /**
     * 根据区域树递归查询设备
     * @param queryDevice 条件
     * @return 设备列表
     */
    List<EmsDevice> selectByAreaTree(QueryDevice queryDevice);

    /**
     * 新增能源设备
     *
     * @param emsDevice 能源设备
     * @return 结果
     */
    int insertEmsDevice(EmsDevice emsDevice);

    /**
     * 修改能源设备
     *
     * @param emsDevice 能源设备
     * @return 结果
     */
    int updateEmsDevice(EmsDevice emsDevice);

    /**
     * 删除能源设备
     *
     * @param id 能源设备主键
     * @return 结果
     */
    int deleteEmsDeviceById(Long id);

    /**
     * 批量删除能源设备
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteEmsDeviceByIds(Long[] ids);
}
